local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local AutomationModule = idTweaks:NewModule('Automation', 'AceConsole-3.0', 'AceEvent-3.0')

AutomationModule.DEBUG = false

function AutomationModule:Debug(...)
  if AutomationModule.DEBUG then
    self:Print(...)
  end
end

function AutomationModule:setDefaults()
  self:Debug('AutomationModule:setDefaults')

  self.defaults = {
    profile = {
      accepting = {
        acceptHearthstoneBind = true,
        acceptPVPRelease = true,
        acceptResurrection = true,
        acceptSummon = true,
      },
      vendors = {
        repairGear = true,
        sellGreyItems = true,
      },
      movement = {
        dismount = true,
      },
    }
  }
end

function AutomationModule:setDatabase()
  self:Debug('AutomationModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Automation', self.defaults)
end

function AutomationModule:setOptions()
  self:Debug('AutomationModule:setOptions')

  self.options = {
    name = 'Automation',
    handler = self,
    type = 'group',
    args = {
      accepting = {
        order = 1,
        name = 'Accepting',
        desc = 'Automatically accept certain offers.',
        handler = self,
        type = 'group',
        args = {
          acceptHearthstoneBind = {
            order = 1,
            name = 'Accept Hearthstone Bind',
            desc = 'Automatically accepts hearthstone binds.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.accepting.acceptHearthstoneBind
            end,
            set = function(_, value)
              self.db.profile.accepting.acceptHearthstoneBind = value
            end,
          },
          acceptPVPRelease = {
            order = 2,
            name = 'Accept PVP Release',
            desc = 'Automatically accepts release on death in PVP.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.accepting.acceptPVPRelease
            end,
            set = function(_, value)
              self.db.profile.accepting.acceptPVPRelease = value
            end,
          },
          acceptResurrection = {
            order = 3,
            name = 'Accept Resurrection',
            desc = 'Automatically accepts resurrections.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.accepting.acceptResurrection
            end,
            set = function(_, value)
              self.db.profile.accepting.acceptResurrection = value
            end,
          },
          acceptSummon = {
            order = 4,
            name = 'Accept Summon',
            desc = 'Automatically accepts summons.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.accepting.acceptSummon
            end,
            set = function(_, value)
              self.db.profile.accepting.acceptSummon = value
            end,
          },
        }
      },
      vendors = {
        order = 2,
        name = 'Vendors',
        desc = 'Automate vendor interactions.',
        handler = self,
        type = 'group',
        args = {
          repairGear = {
            order = 1,
            name = 'Repair gear',
            desc = 'Automatically repairs your gear.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.vendors.repairGear
            end,
            set = function(_, value)
              self.db.profile.vendors.repairGear = value
            end,
          },
          sellGreyItems = {
            order = 2,
            name = 'Sell Grey Items',
            desc = 'Automatically sells trash at vendors.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.vendors.sellGreyItems
            end,
            set = function(_, value)
              self.db.profile.vendors.sellGreyItems = value
            end,
          },
        }
      },
      movement = {
        order = 3,
        name = 'Movement',
        desc = 'Automate movement.',
        handler = self,
        type = 'group',
        args = {
          repairGear = {
            order = 1,
            name = 'Dismount',
            desc = 'Automatically dismounts you when you perform an action that requires you to not be mounted..',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.movement.dismount
            end,
            set = function(_, value)
              self.db.profile.movement.dismount = value
            end,
          },
        }
      }
    },
  }
end

function AutomationModule:registerOptions()
  self:Debug('AutomationModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('Automation', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('Automation', 'Automation', 'idTweaks')
end

function AutomationModule:OnInitialize()
  self:Debug('AutomationModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function AutomationModule:OnEnable()
  self:Debug('AutomationModule:OnEnable')

  self:registerOptions()

  self:RegisterEvent('CONFIRM_BINDER')
  self:RegisterEvent('CONFIRM_SUMMON')
  self:RegisterEvent('MERCHANT_SHOW')
  self:RegisterEvent('PLAYER_DEAD')
  self:RegisterEvent('RESURRECT_REQUEST')
  self:RegisterEvent('UI_ERROR_MESSAGE')
end

function AutomationModule:OnDisable()
  self:Debug('AutomationModule:OnDisable')

  self:UnregisterEvent('CONFIRM_BINDER')
  self:UnregisterEvent('CONFIRM_SUMMON')
  self:UnregisterEvent('MERCHANT_SHOW')
  self:UnregisterEvent('PLAYER_DEAD')
  self:UnregisterEvent('RESURRECT_REQUEST')
  self:UnregisterEvent('UI_ERROR_MESSAGE')
end

function AutomationModule:sellGreyItems()
  self:Debug('AutomationModule:sellGreyItems')

  local link
  for bag = 0, 4 do
    for slot = 0, GetContainerNumSlots(bag) do
      link = GetContainerItemLink(bag, slot)
      if link and select(3, GetItemInfo(link)) == 0 then
        UseContainerItem(bag, slot)
      end
    end
  end
end

function AutomationModule:repairGear()
  self:Debug('AutomationModule:repairGear')

  local cost = GetRepairAllCost()
  local funds = GetMoney()

  if (funds < cost) then
    self:Print('Repair: Insufficient Funds')
  end

  if (cost > 0) then
    RepairAllItems()
    self:Print('Repair: ' .. GetCoinTextureString(cost))
  end
end

function AutomationModule:CONFIRM_BINDER()
  self:Debug('AutomationModule:CONFIRM_BINDER')

  local db = self.db.profile

  if db.accepting.acceptHearthstoneBind == true then
    StaticPopup1Button1:Click()
  end
end

function AutomationModule:CONFIRM_SUMMON()
  self:Debug('AutomationModule:CONFIRM_SUMMON')

  if not self.db.profile.accepting.acceptSummon then return end
  if UnitAffectingCombat('player') then return end

  StaticPopup1:Hide()
  ConfirmSummon()
end

function AutomationModule:MERCHANT_SHOW()
  self:Debug('AutomationModule:MERCHANT_SHOW')

  local db = self.db.profile

  if db.vendors.sellGreyItems == true then
    self:sellGreyItems()
  end

  if CanMerchantRepair() and db.vendors.repairGear == true then
    self:repairGear()
  end
end

function AutomationModule:PLAYER_DEAD()
  self:Debug('AutomationModule:PLAYER_DEAD')

  if not self.db.profile.accepting.acceptPVPRelease then return end
  if UnitAffectingCombat('player') then return end
  if C_DeathInfo.GetSelfResurrectOptions() and #C_DeathInfo.GetSelfResurrectOptions() > 0 then return end

  local inInstance, instanceType = IsInInstance()
  if inInstance and instanceType == 'pvp' then
    RepopMe()
  end
end

function AutomationModule:RESURRECT_REQUEST()
  self:Debug('AutomationModule:RESURRECT_REQUEST')

  if not self.db.profile.accepting.acceptResurrection then return end
  if GetCorpseRecoveryDelay() ~= 0 then return end

  StaticPopup1:Hide()
  AcceptResurrect()
end

local mountErrorMessages = {
  [ERR_ATTACK_MOUNTED] = true,
  [ERR_MOUNT_ALREADYMOUNTED] = true,
  [ERR_NOT_WHILE_MOUNTED] = true,
  [ERR_TAXIPLAYERALREADYMOUNTED] = true,
  [SPELL_FAILED_NOT_MOUNTED] = true,
}

function AutomationModule:UI_ERROR_MESSAGE(_, _, errorMessage)
  self:Debug('AutomationModule:UI_ERROR_MESSAGE', errorMessage)

  if not self.db.profile.movement.dismount then return end
  if not IsMounted() then return end

  if (mountErrorMessages[errorMessage]) then
    Dismount()
  end
end

idTweaks.AutomationModule = AutomationModule
